module bh_vars
IMPLICIT NONE
   Integer , parameter :: MAXOBS = 12686
   Integer , parameter :: MAXBIRTH = 11
   Integer , parameter :: MAXMAR   =  3
   Integer , parameter :: MaxDIV   =  2

Type marr
   Integer :: bdate,edate,mflag,dflag
end type marr 

   Integer , dimension(MAXBIRTH)  :: bir,bflag,bmo,byr,sex,gender
   Integer , dimension(MAXMAR)    :: marmo,maryr
   Integer , dimension(MAXDIV)    :: divmo,divyr
   Type(marr) , dimension(MAXMAR) :: mar

   Integer :: ceb, editf    
   Integer , dimension(1:4) :: editm  !edit flag on male fertility history
end module bh_vars

program birhist
use bh_vars
IMPLICIT NONE
! Date: July 15, 2007 
! program opens annual extracts of birth and marriage history to construct
! complete history for each R. Builds history from last interview.
! Developed for NLS User Conference
! Incorporates information through 2004 interview (March 2007 release)
   
Interface
   Subroutine FERT(caseid)
      use bh_vars
      IMPLICIT NONE
      Integer , intent(in) :: caseid
   end Subroutine FERT
end interface

Interface 
   Subroutine MARRIAGE(rnd,caseid,n_mar)
      use bh_vars
      IMPLICIT NONE
      Integer , intent(in) :: rnd,caseid
      Integer , intent(out) :: n_mar
   end subroutine MARRIAGE
end interface

   Integer , dimension(MAXOBS) :: rflag
   Integer :: rnd,ioc,n,j,nrec,caseid,marnum,totrec,nunit,bunit,munit

   bunit=3
   open(unit=bunit,file="nlsy_birth.dat",STATUS="REPLACE")
   munit=4
   open(unit=munit,file="nlsy_marriage.dat",STATUS="REPLACE")

   rflag(:) = 0
   totrec   = 0
   do rnd=21,4,-1
      nrec = 0
      nunit = rnd + 30  !avoid std out; std error
      select case(rnd)
         case(21)
            open(nunit,file="bh2004.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 2004."
         case(20) 
            open(nunit,file="bh2002.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 2002."
         case(19) 
            open(nunit,file="bh2000.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 2000."
         case(18) 
            open(nunit,file="bh1998.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1998."
         case(17) 
            open(nunit,File="bh1996.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1996."
         case(16) 
            open(nunit,File="bh1994.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1994."
         case(15) 
            open(nunit,File="bh1993.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1993."
         case(14) 
            open(nunit,File="bh1992.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1992."
         case(13) 
            open(nunit,File="bh1991.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1991."
         case(12) 
            open(nunit,File="bh1990.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1990."
         case(11) 
            open(nunit,File="bh1989.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1989."
         case(10) 
            open(nunit,File="bh1988.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1988."
         case(9) 
            open(nunit,File="bh1987.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1987."
         case(8) 
            open(nunit,File="bh1986.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1986."
         case(7) 
            open(nunit,File="bh1985.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1985."
         case(6) 
            open(nunit,File="bh1984.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1984."
         case(5) 
            open(nunit,File="bh1983.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1983."
         case(4) 
            open(nunit,File="bh1982.dat",STATUS="OLD",IOSTAT=ioc,ACTION="READ")
            print *," "
            print *,"Begin reading data for 1982."
      end select

      if (ioc < 0) then
         print *,"Error Opening file for Rnd = ",rnd
         stop
      end if


   ! start with 2004 interview
      do n=1,MAXOBS !loop over all observations
         marnum   = 0
         bmo(:)   = 0
         byr(:)   = 0
         sex(:)   = 0
         marmo(:) = 0
         maryr(:) = 0
         divyr(:) = 0
         select case(rnd)
            case(21)    !2004 in list format
            read(UNIT=nunit,FMT=*,IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,11),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),&
               marmo(3),maryr(3)
            case(20) !2002 
            read(UNIT=nunit,FMT='(I5,11(I2,I4,I2),I2,4I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,11),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),&
               marmo(3),maryr(3)
            case(19) !2000
            read(UNIT=nunit,FMT='(I5,10(I2,I4,I2),I2,4I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,10),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(18) !1998
            read(UNIT=nunit,FMT=*,IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,10),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(16:17) !1994,1996
            read(UNIT=nunit,FMT='(I5,8(I2,I4,I2),I2,4I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,8),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(15) !1993
            read(UNIT=nunit,FMT='(I5,9(3I2),I2,4I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,9),ceb,editf,(editm(j),j=1,3),&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(13:14) !1991,1992
            read(UNIT=nunit,FMT='(I5,9(3I2),I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,9),ceb,&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(10:12) !1988,1989,1990
            read(UNIT=nunit,FMT='(I5,8(3I2),I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,8),ceb,&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(7:9) !1985,1986,1987
            read(UNIT=nunit,FMT='(I5,7(3I2),I2,10I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,7),ceb,&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2),marmo(3),maryr(3)
            case(6) !1984
            read(UNIT=nunit,FMT='(I5,7(3I2),I2,8I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,7),ceb,&
               (marmo(j),maryr(j),divmo(j),divyr(j),j=1,2)
            case(4:5) !1982,1983
            read(UNIT=nunit,FMT='(I5,7(3I2),I2,4I4)',IOSTAT=ioc) &
               caseid,(bmo(j),byr(j),sex(j),j=1,7),ceb,&
               marmo(1),maryr(1),divmo(1),divyr(1),marmo(2),maryr(2)
               ! fill out marriage and divorce arrays 
               divmo(2) = -4; divyr(2) = -4;
               marmo(3) = -4; maryr(3) = -4
            case default
               print *,"ERROR: program designed for only rounds 4-21."
         end select

         if (ioc < 0) then
            exit 
         else if (ioc >= 1) then
            print *,"Error: ",ioc," Reading record ",nrec+1
         end if
         
         if (rnd <= 15) then !convert birth years to 4 digits   
            do j=1,ceb
               if (byr(j) > 0) byr(j) = 1900 + byr(j)
            end do
         end if

         if (rnd <= 17) then !convert marriage years to 4 digits
            do j=1,MAXMAR
               if (maryr(j) > 0) maryr(j) = 1900 + maryr(j)
            end do
            do j=1,MAXMAR-1
               if (divyr(j) > 0) divyr(j) = 1900 + divyr(j)   
            end do
         end if

!         if (nrec < 5) then
         if (caseid == 155) then 
            print '(A,I4,A,I3)',"Caseid= ",caseid," ceb=",ceb
            print *,"Birth history."
            print '(I3,I5,I3)',(bmo(j),byr(j),sex(j),j=1,11)
            print *,"Error codes for fertility."
            print '(5I4)',editf,(editm(j),j=1,3)
            print *,"Marriage History."
            print '(4I5)',(marmo(j),maryr(j),divmo(j),divyr(j),j=1,2)
            print '(2I5)',marmo(3),maryr(3)
            print *,"Error codes for marriage."
            print '(3I4)',(mar(j)%mflag,j=1,3)
            print *,"Error codes for divorce."
            print '(3I4)',(mar(j)%dflag,j=1,3)
            print *," "
         end if

         if (bmo(1) == -5 .OR. rflag(caseid)==1) cycle 

         rflag(caseid) = 1  !case interviewed write history
         if (ceb < 0) then
            print *,"Error in reading file, ceb < 0."
            print *,"Rnd= ",rnd,"  Caseid = ",caseid,"  Ceb = ",ceb
            stop
         end if

!         if (nrec < 25) then
!            print *," "
!            print *,"Caseid ",caseid
!         end if

         CALL FERT(caseid)
         CALL MARRIAGE(rnd,caseid,marnum)

! write case to birth file
         write(unit=bunit,FMT='(I5,2I3,1X,11(I4,2I2))') &
            caseid,rnd,ceb,(bir(j),gender(j),bflag(j),j=1,11)
         write(unit=munit,FMT='(I5,I3,I2,1X,3(2I5,2I4))') &
            caseid,rnd,marnum,&
            (mar(j)%bdate,mar(j)%edate,mar(j)%mflag,mar(j)%dflag,j=1,3)
!         if (caseid == 4) then
!            print *,"finished processing caseid==4"
!            print *,"marnum = ",marnum
!            print *,"Debug:  Stop."
!           stop
!         end if
         nrec = nrec + 1
      end do !maxobs
      close(unit=nunit)
      print *,"Finished Rnd: ",rnd
      print *,"Birth and marriage histories created in rnd: ",nrec
      totrec = totrec + nrec
   end do !rnd
   print *," "
   print *,"Total records created: ",totrec
   close(unit=3)
   close(unit=4)
end program birhist
!-------------------------------------------------------
Subroutine FERT(caseid)
use bh_vars
IMPLICIT NONE
      Integer , intent(in) :: caseid
      Integer :: k
      Integer , SAVE :: bnum=0

      bflag(:) = -9 
      bir(:)   = -9
      gender(:)   = -9
      bnum = bnum + 1
      do k=1,ceb
         gender(k) = sex(k)
         if (bmo(k) < 0 .and. byr(k) < 0) then
                 bflag(k) = 4
                 bir(k) = byr(k)
         else if (bmo(k) < 0 .and. byr(k) > 0) then
                 bflag(k) = 3
                 bir(k) = 12*(byr(k) - 1950) + 6 !assume middle of year
         else if (bmo(k) > 0 .and. byr(k) < 0) then
                 bflag(k) = 2
                 bir(k) = byr(k)
         else if (bmo(k) > 0 .and. byr(k) > 0) then
                 bflag(k) = 0
                 bir(k) = 12*(byr(k) - 1950) + bmo(k)-1
         end if
      end do !k
!      if (caseid == 4) then
!         print *,"Caseid = 4"
!         print '(A,I3)',"Number of births=",ceb
!         if (ceb == 0) then
!            print '(A,I4,A,I4)',"ceb=0, bir(1)= ",bir(1)," bflag(1)= ",bflag(1)
!         else
!            print *,"Coded birth history."
!            do k=1,ceb
!               print '(A,I4,A,2I4)',"k= ",k," bir,bflag ",bir(k),bflag(k)
!            end do !k
!         end if
!         print *," "
!      end if        
end Subroutine FERT
!----------------------------------------
Subroutine MARRIAGE(rnd,caseid,n_mar)
use bh_vars
IMPLICIT NONE
   Integer , intent(in)  :: rnd,caseid
   Integer , intent(out) :: n_mar

   Integer :: k, ks

   mar(:)%mflag = -9
   mar(:)%dflag = -9
   mar(:)%bdate = -9
   mar(:)%edate = -9

   ks = 3
   if (rnd .le. 6) ks = 2
   n_mar = -9
   do
      if (maryr(ks) .eq.  -3)  ks = ks-1
      if (maryr(ks) .ne.  -3  .or. ks==1) exit !do
   end do
   if (maryr(ks) == -999) then !never married
      n_mar = 0
   else if (maryr(ks) == -998) then !once married, never divorced
      n_mar = 1
      CALL assign_date(maryr(1),marmo(1),mar(1)%bdate,mar(1)%mflag)
      mar(1)%edate = 660 !Jan 2005
      mar(1)%dflag = 0
   else if (maryr(ks) == -997) then !once married, once divorced 
      n_mar = 1
      CALL assign_date(maryr(1),marmo(1),mar(1)%bdate,mar(1)%mflag)
      CALL assign_date(divyr(1),divmo(1),mar(1)%edate,mar(1)%dflag)
   else if (maryr(ks) == -996) then !twice married, once divorced
      n_mar = 2
      do k=1,2
         CALL assign_date(maryr(k),marmo(k),mar(k)%bdate,mar(k)%mflag)
      end do
      CALL assign_date(divyr(1),divmo(1),mar(1)%edate,mar(1)%dflag)
      mar(2)%edate = 660 !Jan 2005 never ends
      mar(2)%dflag =   0 
   else if (maryr(ks) == -995) then !twice married, twice divorced
      n_mar = 2
      do k=1,n_mar
         CALL assign_date(maryr(k),marmo(k),mar(k)%bdate,mar(k)%mflag)
         CALL assign_date(divyr(k),divmo(k),mar(k)%edate,mar(k)%dflag)
      end do
   else if (maryr(ks) > -995 .and. maryr(ks) <= 0) then
           print *,"Warning: ks=",ks," maryr(ks) out of range."
           print *," Caseid = ",caseid," maryr(ks) = ",maryr(ks)
   else if (maryr(ks) > 0) then ! three marriages, two divorces
      n_mar = ks
      do k=1,n_mar
         CALL assign_date(maryr(k),marmo(k),mar(k)%bdate,mar(k)%mflag)
         if (k < n_mar) then
            CALL assign_date(divyr(k),divmo(k),mar(k)%edate,mar(k)%dflag)
         else
            mar(n_mar)%edate = 660 !Jan 2005
         end if
      end do
   end if


!   if (caseid == 4) then
!      print *,"Debug caseid == 4, marriage subroutine."
!      print *,"# of mar =",n_mar
!      print *,"     bdate     edate      mflag      dflag"
!      do k=1,3
!        print '(4I10)',mar(k)%bdate,mar(k)%edate,mar(k)%mflag,mar(k)%dflag
!      end do
!   end if

   Contains
!---------------------------------------------------
Subroutine assign_date(yr,mo,edate,qflag)
use bh_vars
IMPLICIT NONE

   Integer , intent(in) :: yr, mo
   Integer , intent(out) :: edate, qflag
         
   qflag = -7

   if (yr < 0 .and. mo < 0) then
      qflag = 4
      edate = yr
   else if (yr > 0 .and. mo < 3) then
      qflag = 3
      edate = 12*(yr - 1950) + 6 ! assume middle of year
   else if (mo > 0 .and. yr < 0) then
      qflag = 2
      mar(k)%edate = yr
   else if (mo > 0 .and. yr > 0) then
      qflag = 0
      edate = 12*(yr - 1950) + mo-1
   end if
end Subroutine assign_date

end Subroutine MARRIAGE
